#!/usr/bin/python
#######################################################################
# Copyright (C) 2008 VMWare, Inc.
# All Rights Reserved
########################################################################
'''Module provides a simple enum class.  Example:
      >>> colors = enum('red', 'orange', 'yellow', 'green', 'blue', 'violet')
      >>> print 'red' in colors
      True
      >>> print colors.red
      red
      >>> colors.orange > colors.red
      True
      >>> colors['blue'] > colors['green']
      True
      >>> print colors
      enum(red, orange, yellow, green, blue, violet)

   Each enum has a list and dict of enumvalues, which is
   a string-derived class holding the enumerated value, and
   allows comparisons based on the enumerated value.
'''

class enumvalue(str):
   def __new__(self, value, othervalue):
      return str.__new__(self, value)
   
   def __init__(self, value, num):
      str.__init__(self, value)
      self.num = num

   def __cmp__(self, other):
      if hasattr(other, 'num'):
         return cmp(self.num, other.num)
      else:
         return str.__cmp__(self, other)

   #
   # __lt__ and __gt__ are needed to override
   # the default methods from the str class.
   # Apparently overriding __cmp__ is not enough.
   #
   def __lt__(self, other):
      return self.__cmp__(other) == -1
   def __gt__(self, other):
      return self.__cmp__(other) == 1

class enum(object):
   def __init__(self, *values):
      self._values = []
      self._enums = {}
      for i in range(len(values)):
         self._enums[values[i]] = enumvalue(values[i], i)
         self._values.append(enumvalue(values[i], i))
   def __getattr__(self, name):
      if name in self._enums: return self._enums[name]
      raise AttributeError(name)
   def __getitem__(self, name):
      return self._enums[name]
   def __iter__(self):
      for x in self._values: yield x
   def __str__(self):
      return 'enum(%s)' % ', '.join(self._values)
